#!/usr/bin/env python3

from pathlib import Path
import argparse
import os
import subprocess
import sys
import typing as T

from mesonbuild.mesonlib import version_compare

modules = [
    # fully typed submodules
    # 'mesonbuild/ast',
    'mesonbuild/cmake',
    'mesonbuild/compilers',
    'mesonbuild/dependencies',
    'mesonbuild/interpreterbase',
    'mesonbuild/linkers',
    'mesonbuild/scripts',
    'mesonbuild/wrap',

    # specific files
    'mesonbuild/arglist.py',
    # 'mesonbuild/coredata.py',
    'mesonbuild/envconfig.py',
    'mesonbuild/interpreter/interpreterobjects.py',
    'mesonbuild/mcompile.py',
    'mesonbuild/mdevenv.py',
    'mesonbuild/mesonlib/platform.py',
    'mesonbuild/mesonlib/universal.py',
    'mesonbuild/minit.py',
    'mesonbuild/minstall.py',
    'mesonbuild/mintro.py',
    'mesonbuild/mlog.py',
    'mesonbuild/modules/fs.py',
    'mesonbuild/modules/unstable_rust.py',
    'mesonbuild/modules/qt.py',
    'mesonbuild/mparser.py',
    'mesonbuild/msetup.py',
    'mesonbuild/mtest.py',
    'mesonbuild/optinterpreter.py',
    'mesonbuild/programs.py',

    'run_custom_lint.py',
    'run_mypy.py',
    'run_project_tests.py',
    'run_single_test.py',
    'tools'
]

if os.name == 'posix':
    modules.append('mesonbuild/mesonlib/posix.py')
elif os.name == 'nt':
    modules.append('mesonbuild/mesonlib/win32.py')

def check_mypy() -> None:
    try:
        import mypy
    except ImportError:
        print('Failed import mypy')
        sys.exit(1)
    from mypy.version import __version__ as mypy_version
    if not version_compare(mypy_version, '>=0.812'):
        print('mypy >=0.812 is required, older versions report spurious errors')
        sys.exit(1)

def main() -> int:
    check_mypy()

    root = Path(__file__).absolute().parent
    args = []  # type: T.List[str]

    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('-p', '--pretty', action='store_true', help='pretty print mypy errors')
    parser.add_argument('-C', '--clear', action='store_true', help='clear the terminal before running mypy')

    opts = parser.parse_args()
    if opts.pretty:
        args.append('--pretty')

    if opts.clear:
        print('\x1bc', end='', flush=True)

    print('Running mypy (this can take some time) ...')
    p = subprocess.run(
        [sys.executable, '-m', 'mypy'] + args + modules,
        cwd=root,
    )
    return p.returncode

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3

# Copyright 2019 The Meson development team
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
'''
  Generates release notes for new releases of Meson build system
'''
import subprocess
import sys
import os
from glob import glob

RELNOTE_TEMPLATE = '''---
title: Release {}
short-description: Release notes for {}
...

# New features

'''


def add_to_sitemap(from_version, to_version):
    '''
       Adds release note entry to sitemap.txt.
    '''
    sitemapfile = '../sitemap.txt'
    s_f = open(sitemapfile, encoding='utf-8')
    lines = s_f.readlines()
    s_f.close()
    with open(sitemapfile, 'w', encoding='utf-8') as s_f:
        for line in lines:
            if 'Release-notes' in line and from_version in line:
                new_line = line.replace(from_version, to_version)
                s_f.write(new_line)
            s_f.write(line)

def generate(from_version, to_version):
    '''
       Generate notes for Meson build next release.
    '''
    ofilename = f'Release-notes-for-{to_version}.md'
    with open(ofilename, 'w', encoding='utf-8') as ofile:
        ofile.write(RELNOTE_TEMPLATE.format(to_version, to_version))
        for snippetfile in glob('snippets/*.md'):
            snippet = open(snippetfile, encoding='utf-8').read()
            ofile.write(snippet)
            if not snippet.endswith('\n'):
                ofile.write('\n')
            ofile.write('\n')
            subprocess.check_call(['git', 'rm', snippetfile])
    subprocess.check_call(['git', 'add', ofilename])
    add_to_sitemap(from_version, to_version)

if __name__ == '__main__':
    if len(sys.argv) != 3:
        print(sys.argv[0], 'from_version to_version')
        sys.exit(1)
    FROM_VERSION = sys.argv[1]
    TO_VERSION = sys.argv[2]
    os.chdir('markdown')
    generate(FROM_VERSION, TO_VERSION)



#!/usr/bin/env python3

from pathlib import Path
import typing as T

root = Path(__file__).absolute().parent
mesonbuild = root / 'mesonbuild'

whitelist = ['mesonbuild/', 'run_', 'ci/', 'tools/', 'docs/']

def check_missing_encoding(lines: T.List[str], path: str) -> int:
    errors = 0
    functions = ['read_text', 'write_text', 'open']
    for num, line in enumerate(lines):
        for func in functions:
            l = line

            # Skip ignored lines
            if '[ignore encoding]' in l:
                continue

            # Do we have a match?
            loc = l.find(func + '(')
            if loc < 0:
                continue
            if loc > 0 and ord(l[loc-1].lower()) in [*range(ord('a'), ord('z')), *range(ord('0'), ord('9')), '_']:
                continue
            loc += len(func) + 1
            # Some preprocessign to make parsing easier
            l = l[loc:]
            l = l.replace(' ', '')
            l = l.replace('\t', '')
            l = l.replace('\n', '')
            l = l.replace('\'', '"')

            # Parameter begin
            args = ''
            b_open = 1
            while l:
                c = l[0]
                l = l[1:]
                if c == ')':
                    b_open -= 1
                if b_open == 0:
                    break
                elif b_open == 1:
                    args += c
                if c == '(':
                    b_open += 1

            binary_modes = ['rb', 'br', 'r+b', 'wb', 'bw', 'ab', 'ba']
            is_binary = any([f'"{x}"' in args for x in binary_modes])
            if 'encoding=' not in args and not (func == 'open' and is_binary):
                location = f'\x1b[33;1m[\x1b[0;1m{path}:{num+1}\x1b[33m]\x1b[0m'
                #print(f'{location:<64}: \x1b[31;1mERROR:\x1b[0m Missing `encoding=` parameter in "{line.strip()}"')
                print(f'{location:<72}: \x1b[31;1mERROR:\x1b[0m Missing `encoding=` parameter in `{func}` call')
                errors += 1
    return errors

def main() -> int:
    print('Scanning mesonbuild...')
    errors = 0
    for i in sorted(root.glob('**/*.py')):
        raw = i.read_text(encoding='utf-8')
        lines = raw.splitlines()
        filename = i.relative_to(root).as_posix()

        if not any([filename.startswith(x) for x in whitelist]):
            continue

        errors += check_missing_encoding(lines, filename)
    print(f'Found {errors} errors while scanning mesonbuild')
    return 0 if errors == 0 else 1

if __name__ == '__main__':
    raise SystemExit(main())
